/* This file is part of Eclat.
   Copyright (C) 2012 Sergey Poznyakoff.
 
   Eclat is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.
 
   Eclat is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with Eclat.  If not, see <http://www.gnu.org/licenses/>. */

#include "eclat.h"

static void
parse_options(const char *progname, const char *docstring,
              int argc, char *argv[], int *index)
{
	generic_proginfo->args_doc = "ID [ID...]";
	return generic_parse_options(progname, docstring, argc, argv, index);
}

static int
start_stop_instance(CURL *curl, const char *action, int argc, char **argv)
{
	int i;
	struct ec2_query *q;
	char buf[128], *bend, *url;
	size_t bs;
	CURLcode res;

	if (argc == 0)
		die(EX_USAGE, "no instance ids");
	translate_ids(argc, argv, "InstanceId");
	
	q = eclat_query_create(use_ssl ? EC2_QF_HTTPS : 0, endpoint, "/");
	eclat_query_add_param(q, "Action", action);

	strcpy(buf, "InstanceId.");
	bend = buf + strlen(buf);
	bs = sizeof(buf) - strlen(buf);
	for (i = 0; i < argc; i++) {
		snprintf(bend, bs, "%lu", (unsigned long)(i + 1));
		eclat_query_add_param(q, buf, argv[i]);
	}

	eclat_query_add_param(q, "AWSAccessKeyId", access_key);
	
	eclat_query_signature(q, secret_key);
	url = eclat_query_to_url(q, NULL);

	debug(ECLAT_DEBCAT_MAIN, 1, ("using URL: %s", url));
	curl_easy_setopt(curl, CURLOPT_URL, url);

	free(url);
	eclat_query_free(q);

	if (dry_run_mode)
		debug(ECLAT_DEBCAT_MAIN, 1, ("not sending request"));
	else {
		res = curl_easy_perform(curl);
		
		if (res != CURLE_OK)
			err("CURL: %s", curl_easy_strerror(res));
	}
	
	return 0;	
}

int
eclat_start_instance(CURL *curl, int argc, char **argv)
{
	int i;

	parse_options("eclat start-instances",
		      "Start named instances",
		      argc, argv, &i);

	debug(ECLAT_DEBCAT_MAIN, 1, ("starting instances"));
	return start_stop_instance(curl, "StartInstances", argc - i, argv + i);
}

int
eclat_stop_instance(CURL *curl, int argc, char **argv)
{
	int i;

	parse_options("eclat stop-instances",
		      "Stop named instances",
		      argc, argv, &i);

	debug(ECLAT_DEBCAT_MAIN, 1, ("stopping instances"));
	return start_stop_instance(curl, "StopInstances", argc - i, argv + i);
}

int
eclat_reboot_instance(CURL *curl, int argc, char **argv)
{
	int i;

	parse_options("eclat reboot-instances",
		      "Reboot named instances",
		      argc, argv, &i);

	debug(ECLAT_DEBCAT_MAIN, 1, ("rebooting instances"));
	return start_stop_instance(curl, "RebootInstances", argc - i, argv + i);
}
